# -*- coding: utf-8 -*-
import os
import sys
import json
import yaml
import time
import itertools
import threading
import traceback

from scalrctl import click, defaults, settings


def read_spec(api_level, ext='json'):
    """
    Reads Scalr specification file, json or yaml.
    """

    spec_path = os.path.join(defaults.CONFIG_DIRECTORY,
                             '{}.{}'.format(api_level, ext))

    if os.path.exists(spec_path):
        with open(spec_path, 'r') as fp:
            spec_data = fp.read()

        if ext == 'json':
            return json.loads(spec_data)
        elif ext == 'yaml':
            return yaml.safe_load(spec_data)
    else:
        msg = "Scalr specification file '{}' does  not exist, " \
              "try to run 'scalr-ctl update'.".format(spec_path)
        raise click.ClickException(msg)


def read_routes():
    if os.path.exists(defaults.ROUTES_PATH):
        with open(defaults.ROUTES_PATH, 'r') as fp:
            api_routes = fp.read()
        return json.loads(api_routes)


def read_scheme():
    with open(os.path.join(os.path.dirname(__file__),
                           'scheme/scheme.json')) as fp:
        return json.load(fp)


def read_config(profile=None):
    confpath = os.path.join(
        defaults.CONFIG_DIRECTORY,
        '{}.yaml'.format(profile)
    ) if profile else defaults.CONFIG_PATH

    if os.path.exists(confpath):
        with open(confpath, 'r') as fp:
            return yaml.safe_load(fp)


def warning(*messages):
    """
    Prints the warning message(s) to stderr.

    :param tuple messages: The list of the warning messages.
    :rtype: None
    """
    color = 'yellow' if settings.colored_output else None
    for index, message in enumerate(messages or [], start=1):
        index = index if len(messages) > 1 else None
        code = message.get('code') or ''
        text = message.get('message') or ''
        click.secho("Warning{index}{code} {text}".format(
            index=' {}:'.format(index) if index else ':',
            code=' {}:'.format(code) if code else '',
            text=text
        ), err=True, fg=color)


def debug(msg):
    if settings.debug_mode:
        click.secho("DEBUG: {}".format(msg),
                    fg='green' if settings.colored_output else None)


def reraise(message):
    import sys
    exc_info = sys.exc_info()
    if isinstance(exc_info[1], click.ClickException):
        exc_class = exc_info[0]
    else:
        exc_class = click.ClickException
    debug(traceback.format_exc())
    message = str(message)
    if not settings.debug_mode:
        message = "{}, use '--debug' option for details.".format(message)
    raise click.ClickException(message)


class _spinner(object):

    @staticmethod
    def draw(event):
        if settings.colored_output:
            cursor = itertools.cycle('|/-\\')
            while not event.isSet():
                sys.stdout.write(next(cursor))
                sys.stdout.flush()
                time.sleep(0.1)
                sys.stdout.write('\b')
            sys.stdout.write(' ')
            sys.stdout.flush()

    def __init__(self):
        self.event = threading.Event()
        self.thread = threading.Thread(target=_spinner.draw,
                                       args=(self.event,))
        self.thread.daemon = True

    def __enter__(self):
        self.thread.start()

    def __exit__(self, type, value, traceback):
        self.event.set()
        self.thread.join()






__author__ = 'Dmitriy Korsakov'
__doc__ = 'Manage FarmRoles'

import json
import copy

from scalrctl import commands
from scalrctl import click

from scalrctl import request, settings
from scalrctl.commands import farm


class LaunchServer(commands.SimplifiedAction):

    post_template = {}

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        kv = {"import-data": {}}
        kv.update(kwargs)
        arguments, kw = super(LaunchServer, self).pre(*args, **kv)
        return arguments, kw


class FarmRoleClone(commands.SimplifiedAction):

    epilog = "Example: scalr-ctl farm-roles clone --farmRoleId <ID> --alias <ALIAS> [--farmId <FarmID>]"

    post_template = {
        "cloneFarmRoleRequest": {"name": ""}
    }

    _table_columns = [u'alias', u'cloudPlatform', u'farm.id', u'id', u'instanceType.id',
                      u'role.id']

    def get_options(self):
        alias_hlp = "New alias name for the FarmRole being cloned."
        alias = click.Option(('--alias', 'alias'), required=True, help=alias_hlp)

        farm_hlp = "Identifier of the Farm to clone into. If omitted, current Farm is a target to clone the FarmRole."
        farm_id = click.Option(('--farmId', 'farm_id'), required=False, help=farm_hlp)

        options = [alias, farm_id]
        options.extend(super(FarmRoleClone, self).get_options())
        return options

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        name = kwargs.pop("alias", None)
        farm_id = kwargs.pop("farm_id", None)
        post_data = copy.deepcopy(self.post_template)
        post_data["cloneFarmRoleRequest"]["name"] = name
        if farm_id:
            post_data["cloneFarmRoleRequest"]["farm"] = {}
            post_data["cloneFarmRoleRequest"]["farm"]["id"] = farm_id
        kv = {"import-data": post_data}
        kv.update(kwargs)
        arguments, kw = super(FarmRoleClone, self).pre(*args, **kv)
        return arguments, kw


class FarmRoleCreateFromTemplate(farm.FarmCreateFromTemplate):

    def pre(self, *args, **kwargs):
        """
        before request is made
        """
        kwargs = self._apply_arguments(**kwargs)
        stdin = kwargs.pop('stdin', None)
        kwargs["FarmRoleTemplate"] = self._read_object() if stdin else self._edit_example()
        return args, kwargs

    def _edit_example(self):
        commentary = \
            '''# The body must be a valid FarmRoleTemplate object.
#
# Type your FarmRoleTemplate object below this line. The above text will not be sent to the API server.'''
        text = click.edit(commentary)
        if text:
            raw_object = "".join([line for line in text.splitlines()
                                  if not line.startswith("#")]).strip()
        else:
            raw_object = ""
        return json.loads(raw_object)
